"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var interfaces_1 = require("./interfaces");
var utils_1 = require("../utils");
var Play = (function (_super) {
    __extends(Play, _super);
    function Play(url, name) {
        if (url === void 0) { url = ""; }
        if (name === void 0) { name = "action"; }
        var _this = _super.call(this, {
            type: "PLAY",
            name: name,
            comments: "",
            params: url.indexOf("http") === 0 ? {
                id: interfaces_1.CoreAction.genId("play"),
                url: url,
                stop_on_key: "false",
                repeatMax: "0"
            } : {
                id: interfaces_1.CoreAction.genId("dtmf"),
                dtmf: url,
                dtmf_delay: 250,
                dtmf_dash_delay: 250,
            }
        }) || this;
        _this.name = name;
        _this._paramMap = {
            id: "id",
            url: "url",
            stop_on_key: "stopOnKeyPressed",
            dtmf: "dtmf",
            dtmf_delay: "digitDelay",
            dtmf_dash_delay: "pauseDelay",
            repeatMax: "repeatMax"
        };
        _this._repeat = 1;
        _this._parseVars = true;
        _this._injectCCML = "";
        _this.type = interfaces_1.Type.Play;
        return _this;
    }
    Object.defineProperty(Play.prototype, "id", {
        get: function () {
            return this.getParam("id");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "repeat", {
        get: function () {
            return this._repeat;
        },
        set: function (num) {
            this._repeat = Math.min(Math.max(1, +num), 10);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "parseVars", {
        get: function () {
            return this._parseVars;
        },
        set: function (val) {
            this._parseVars = val;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "stopOnKey", {
        get: function () {
            return this.getParam("stop_on_key") === "true";
        },
        set: function (stop) {
            this.setParam("stop_on_key", stop ? "true" : "false");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "dtmfDelay", {
        get: function () {
            return +this.getParam("dtmf_delay");
        },
        set: function (v) {
            this.setParam("dtmf_delay", String(v));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "dtmfDashDelay", {
        get: function () {
            return +this.getParam("dtmf_dash_delay");
        },
        set: function (v) {
            this.setParam("dtmf_dash_delay", String(v));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Play.prototype, "injectCCML", {
        get: function () {
            return this._injectCCML;
        },
        set: function (c) {
            this._injectCCML = c || "";
        },
        enumerable: true,
        configurable: true
    });
    Play.prototype.toCCML = function (filter) {
        return this.getParam("url") ? this.playToCCML(filter) : this.dtmfToCCML(filter);
    };
    Play.prototype.playToCCML = function (filter) {
        var ccml = [];
        if (this._parseVars && filter !== "says") {
            var resp = utils_1.VarParser.parseVars(this.getParam("url"));
            this.setParam("url", resp[0]);
            ccml = ccml.concat(resp[1]);
        }
        if (filter !== "sets") {
            if (this.repeat > 1) {
                delete this._paramMap.id;
                var action = "<PLAY" + _super.prototype.toCCML.call(this, this._paramMap) + " " + (this.injectCCML ? "/>" + this.injectCCML + "<PLAY" : "") + "/>";
                for (var i = 0; i < this.repeat; i++) {
                    ccml.push(action);
                }
            }
            else {
                ccml.push("<PLAY" + _super.prototype.toCCML.call(this, this._paramMap) + " " + (this.injectCCML ? "/>" + this.injectCCML + "<PLAY" : "") + "/>");
            }
        }
        return ccml.join("");
    };
    Play.prototype.dtmfToCCML = function (filter) {
        var ccml = [];
        this.setParam("dtmf", this.getParam("dtmf").replace(/-/g, ","));
        if (this._parseVars && filter !== "says") {
            var resp = utils_1.VarParser.parseVars(this.getParam("dtmf"));
            this.setParam("dtmf", resp[0]);
            ccml = ccml.concat(resp[1]);
        }
        if (filter !== "sets") {
            if (this.repeat > 1) {
                delete this._paramMap.id;
                var action = "<SENDDTMF" + _super.prototype.toCCML.call(this, this._paramMap) + "/>";
                for (var i = 0; i < this.repeat; i++) {
                    ccml.push(action);
                }
            }
            else {
                ccml.push("<SENDDTMF" + _super.prototype.toCCML.call(this, this._paramMap) + "/>");
            }
        }
        return ccml.join("");
    };
    return Play;
}(interfaces_1.CoreAction));
exports.Play = Play;
//# sourceMappingURL=Play.js.map