"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var interfaces_1 = require("./interfaces");
var utils_1 = require("../utils");
var QueueEvent;
(function (QueueEvent) {
    QueueEvent[QueueEvent["OFFER"] = 0] = "OFFER";
    QueueEvent[QueueEvent["POLL"] = 1] = "POLL";
    QueueEvent[QueueEvent["POLL_CONNECT"] = 2] = "POLL_CONNECT";
    QueueEvent[QueueEvent["REMOVE"] = 3] = "REMOVE";
})(QueueEvent = exports.QueueEvent || (exports.QueueEvent = {}));
exports.QueueOnEvent = {
    OnEnter: "on_enter",
    OnEnterError: "on_enter_error",
    OnEnterErrorMaxCalls: "on_enter_error_max_calls"
};
var Queue = (function (_super) {
    __extends(Queue, _super);
    function Queue(queueName, name) {
        if (queueName === void 0) { queueName = "myQueue"; }
        if (name === void 0) { name = "action"; }
        var _this = _super.call(this, {
            type: "QUEUE",
            name: name,
            comments: "",
            params: {
                id: interfaces_1.CoreAction.genId("queue"),
                event: "enqueue",
                max_calls: null,
                async: "false",
            },
            events: (_a = {},
                _a[exports.QueueOnEvent.OnEnter] = [],
                _a[exports.QueueOnEvent.OnEnterError] = [],
                _a[exports.QueueOnEvent.OnEnterErrorMaxCalls] = [],
                _a)
        }) || this;
        _this.queueName = queueName;
        _this.name = name;
        _this._paramMap = {
            id: "id",
            event: "control",
            max_calls: "maxCalls",
            async: "async",
        };
        _this._holdAudio = [""];
        _this._event = QueueEvent.OFFER;
        _this.type = interfaces_1.Type.Queue;
        return _this;
        var _a;
    }
    Object.defineProperty(Queue.prototype, "id", {
        get: function () {
            return this.getParam("id");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Queue.prototype, "event", {
        get: function () {
            return this._event;
        },
        set: function (evt) {
            this._event = evt;
            this.setParam("event", evt === (QueueEvent.POLL || QueueEvent.POLL_CONNECT) ? "dequeue" : evt === QueueEvent.REMOVE ? "removeMe" : "enqueue");
            if (this.getParam("event") !== "enqueue") {
                this.setParam("async", "true");
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Queue.prototype, "holdAudio", {
        get: function () {
            if (this._holdAudio.length === 0) {
                return false;
            }
            else if (this._holdAudio[0] === "") {
                return true;
            }
            return this._holdAudio.slice(0);
        },
        set: function (url) {
            if (url === true) {
                this._holdAudio = [""];
            }
            else if (url === false) {
                this._holdAudio = [];
            }
            else {
                this._holdAudio = [].concat(url);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Queue.prototype, "async", {
        get: function () {
            return this.getParam("async") === "true";
        },
        set: function (val) {
            this.setParam("async", val ? "true" : "false");
            if (this.getParam("event") !== "enqueue") {
                this.setParam("async", "true");
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Queue.prototype, "maxCalls", {
        get: function () {
            return this.getParam("max_calls") ? parseInt(this.getParam("max_calls"), 10) : undefined;
        },
        set: function (val) {
            this.setParam("max_calls", typeof val === "number" ? String(val) : "");
        },
        enumerable: true,
        configurable: true
    });
    Queue.prototype.toCCML = function () {
        var id = interfaces_1.CoreAction.genId();
        var ccml = "";
        var p = utils_1.VarParser.parseVars(this.queueName);
        this.queueName = p[0];
        ccml += p[1].join("");
        if (this._event === QueueEvent.OFFER && this.events[exports.QueueOnEvent.OnEnter].length > 0) {
            var params = _super.prototype.toCCML.call(this, this._paramMap)
                .replace("id=\"", "id = \"2_")
                .replace("async=\"false\"", "async=\"true\"");
            ccml += "<CALLQUEUE errorActionId=\"" + id + "_errorHandler\"" + params + "><![CDATA[" + this.queueName + "]]></CALLQUEUE>\n                    " + this.events[exports.QueueOnEvent.OnEnter].map(function (e) { return e.toCCML(); }).join("");
        }
        if (this._event === QueueEvent.OFFER && (this.events[exports.QueueOnEvent.OnEnterErrorMaxCalls].length > 0 || this.events[exports.QueueOnEvent.OnEnterError].length > 0)) {
            ccml += "<CALLQUEUE errorActionId=\"" + id + "_errorHandler\"" + _super.prototype.toCCML.call(this, this._paramMap) + "><![CDATA[" + this.queueName + "]]>";
            this._holdAudio.forEach(function (url) { return ccml += url !== "" ? "<HOLDMUSIC url=\"" + url + "\"/>" : "<HOLDMUSIC/>"; });
            ccml += "</CALLQUEUE>";
            ccml += "\n                <WAIT condition=\"true\" nextActionId=\"" + id + "_done\"/>\n                <CONDITION id=\"" + id + "_errorHandler\">\n                    <WAIT error=\"queueFull\" nextActionId=\"" + id + "_onEnterErrorMaxCalls\"/>\n                    <WAIT error=\"any\" nextActionId=\"" + id + "_onEnterError\"/>\n                </CONDITION>\n                <BOOKMARK id=\"" + id + "_onEnterErrorMaxCalls\"/>\n                    " + this.events[exports.QueueOnEvent.OnEnterErrorMaxCalls].map(function (e) { return e.toCCML(); }).join("") + "\n                <BOOKMARK id=\"" + id + "_onEnterError\"/>\n                    " + this.events[exports.QueueOnEvent.OnEnterError].map(function (e) { return e.toCCML(); }).join("") + "\n                <BOOKMARK id=\"" + id + "_done\"/>\n            ";
        }
        else if (this._event === QueueEvent.POLL_CONNECT) {
            ccml += "\n                <CALLQUEUE errorActionId=\"" + id + "_onEnterError\" control=\"dequeue\" async=\"true\"><![CDATA[" + this.queueName + "]]></CALLQUEUE>\n                <CONDITION>\n                    <WAIT condition=\"${queueEntry} !== null\" />\n                    <WAIT condition=\"true\" nextActionId=\"" + id + "_onEnterError\" />\n                </CONDITION>\n                <SET varName=\"___lastQueuePollConnectCall___\" export=\"true\" evaluate=\"true\"><![CDATA[ JSON.stringify({id:${queueEntry.callId}, no:${queueEntry.phoneNo}}) ]]></SET>\n                <CALLTRANSFER errorActionId=\"" + id + "_onEnterError\" async=\"true\">${queueEntry}</CALLTRANSFER>\n                <WAIT callTransfer=\"routed\"/>\n                " + this.events[exports.QueueOnEvent.OnEnter].map(function (e) { return e.toCCML(); }).join("") + "\n                <CONDITION>\n                    <WAIT callTransfer=\"ended\" nextActionId=\"" + id + "_done\"/>\n                    <TIMER seconds=\"14400\" nextActionId=\"" + id + "_done\" />\n                </CONDITION>\n                <WAIT nextActionId=\"" + id + "_done\"/>\n                <BOOKMARK id=\"" + id + "_onEnterError\"/>\n                    " + this.events[exports.QueueOnEvent.OnEnterError].map(function (e) { return e.toCCML(); }).join("") + "\n                <BOOKMARK id=\"" + id + "_done\" />\n            ";
        }
        else if (this._event === QueueEvent.POLL) {
            ccml += "\n                <CALLQUEUE " + _super.prototype.toCCML.call(this, this._paramMap) + "><![CDATA[" + this.queueName + "]]></CALLQUEUE>\n                <CONDITION>\n                    <WAIT condition=\"${queueEntry} !== null\" />\n                    <WAIT condition=\"true\" nextActionId=\"" + id + "_errorHandler\" />\n                </CONDITION>\n                <SET varName=\"___lastQueuePollConnectCall___\" export=\"true\" evaluate=\"true\"><![CDATA[ JSON.stringify({id:${queueEntry.callId}, no:${queueEntry.phoneNo}}) ]]></SET>\n                " + this.events[exports.QueueOnEvent.OnEnter].map(function (e) { return e.toCCML(); }).join("") + "\n                <WAIT nextActionId=\"" + id + "_done\" />\n                <BOOKMARK id=\"" + id + "_errorHandler\"/>\n                    " + this.events[exports.QueueOnEvent.OnEnterError].map(function (e) { return e.toCCML(); }).join("") + "\n                <BOOKMARK id=\"" + id + "_done\" />\n            ";
        }
        else {
            ccml += "<CALLQUEUE " + _super.prototype.toCCML.call(this, this._paramMap) + "><![CDATA[" + this.queueName + "]]>";
            if (this._event === QueueEvent.OFFER) {
                this._holdAudio.forEach(function (url) { return ccml += url !== "" ? "<HOLDMUSIC url=\"" + url + "\"/>" : "<HOLDMUSIC/>"; });
            }
            ccml += "</CALLQUEUE><BOOKMARK id=\"" + id + "_errorHandler\"/>";
        }
        if (this._event === QueueEvent.OFFER && this.async === false) {
            ccml += "<TIMER milliSeconds=\"1000\" waitForIdle=\"false\"/>";
        }
        return ccml;
    };
    return Queue;
}(interfaces_1.CoreAction));
exports.Queue = Queue;
//# sourceMappingURL=Queue.js.map