"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var interfaces_1 = require("./interfaces");
var utils_1 = require("../utils");
exports.RecordMessageOnEvent = {
    OnError: "on_error",
    OnSave: "on_save",
};
var RecordMessage = (function (_super) {
    __extends(RecordMessage, _super);
    function RecordMessage(action, group, name) {
        if (action === void 0) { action = "START"; }
        if (group === void 0) { group = "$PN+{ani}"; }
        if (name === void 0) { name = "recmsg_" + interfaces_1.CoreAction.genId(); }
        var _this = _super.call(this, {
            type: "RECORDMESSAGE",
            name: name,
            comments: "",
            params: {
                id: interfaces_1.CoreAction.genId("recmsg"),
                play_beep: "true",
                finish_on_key: "any",
                trim_silence: "true",
                timeout: "5",
                max_recording_time: "30",
            },
            events: (_a = {},
                _a[exports.RecordMessageOnEvent.OnError] = [],
                _a[exports.RecordMessageOnEvent.OnSave] = [],
                _a),
        }) || this;
        _this.action = action;
        _this.group = group;
        _this.name = name;
        _this._paramMap = {
            id: "id",
            play_beep: "playBeep",
            finish_on_key: "endKey",
            trim_silence: "trimSilence",
            timeout: "maxSilenceTime",
            max_recording_time: "maxRecordingTime",
        };
        _this.type = interfaces_1.Type.RecordMessage;
        action = action.toUpperCase();
        return _this;
        var _a;
    }
    Object.defineProperty(RecordMessage.prototype, "id", {
        get: function () {
            return this.getParam("id");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordMessage.prototype, "finishOnKey", {
        get: function () {
            var key = this.getParam("finish_on_key");
            switch (key) {
                case "hash":
                case "pound": return "#";
                case "star": return "*";
                case "": return "off";
                default: return key;
            }
        },
        set: function (key) {
            key = String(key).toLowerCase();
            if (key === "#") {
                key = "pound";
            }
            else if (key === "*") {
                key = "star";
            }
            else if (key === "off") {
                key = "";
            }
            this.setParam("finish_on_key", key);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordMessage.prototype, "playBeep", {
        get: function () {
            return this.getParam("play_beep") === "true";
        },
        set: function (key) {
            this.setParam("play_beep", String(key));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordMessage.prototype, "trimSilence", {
        get: function () {
            return this.getParam("trim_silence") === "true";
        },
        set: function (key) {
            this.setParam("trim_silence", String(key));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordMessage.prototype, "maxRecordingTime", {
        get: function () {
            return +this.getParam("max_recording_time");
        },
        set: function (num) {
            this.setParam("max_recording_time", String(Math.min(60 * 60, Math.max(1, num))));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RecordMessage.prototype, "timeout", {
        get: function () {
            return +this.getParam("timeout");
        },
        set: function (s) {
            this.setParam("timeout", String(Math.min(60, Math.max(1, s))));
        },
        enumerable: true,
        configurable: true
    });
    RecordMessage.prototype.toCCML = function () {
        switch (this.action) {
            case "START":
                return this.startCCML();
            case "PLAY":
                return this.playCCML();
            case "DISCARD":
                return this.discardCCML();
            case "SAVE":
                return this.saveCCML();
            default:
                return "<SAY>The record message action of " + this.action + " is invalid.</SAY>";
        }
    };
    RecordMessage.prototype.startCCML = function () {
        var id = interfaces_1.CoreAction.genId();
        var ccml = [];
        var groupVars = utils_1.VarParser.parseVars(this.group);
        ccml.push("<SET varName=\"___recordMessageName___\" value=\"" + this.name + "\" export=\"false\"/>");
        ccml.push.apply(ccml, groupVars[1]);
        ccml.push("<SET varName=\"___recordMessageGroup___\" value=\"" + groupVars[0] + "\" export=\"false\"/>");
        ccml.push("<RECORD voiceMailName=\"___recordMessageGroup___\" " + _super.prototype.toCCML.call(this, this._paramMap) + " errorActionId=\"" + id + "_onError\"/>");
        ccml.push("<WAIT nextActionId=\"" + id + "_onDone\"/>");
        ccml.push("<BOOKMARK id=\"" + id + "_onError\"/>");
        ccml.push.apply(ccml, this.events[exports.RecordMessageOnEvent.OnError].map(function (a) { return a.toCCML(); }));
        ccml.push("<BOOKMARK id=\"" + id + "_onDone\"/>");
        return ccml.join("");
    };
    RecordMessage.prototype.playCCML = function () {
        var id = interfaces_1.CoreAction.genId();
        var ccml = [];
        ccml.push("<PLAY vfileName=\"${recording}\" stopOnKeyPressed=\"true\" errorActionId=\"" + id + "_onError\"/>");
        ccml.push("<WAIT nextActionId=\"" + id + "_onDone\"/>");
        ccml.push("<BOOKMARK id=\"" + id + "_onError\"/>");
        ccml.push.apply(ccml, this.events[exports.RecordMessageOnEvent.OnError].map(function (a) { return a.toCCML(); }));
        ccml.push("<BOOKMARK id=\"" + id + "_onDone\"/>");
        return ccml.join("");
    };
    RecordMessage.prototype.discardCCML = function () {
        var ccml = [];
        ccml.push("<SET varName=\"___recordMessageId___\" value=\"\" export=\"false\"/>");
        ccml.push("<SET varName=\"___recordMessageName___\" value=\"\" export=\"false\"/>");
        ccml.push("<SET varName=\"___recordMessageGroup___\" value=\"\" export=\"false\"/>");
        return ccml.join("");
    };
    RecordMessage.prototype.saveCCML = function () {
        var id = interfaces_1.CoreAction.genId();
        var ccml = [];
        var groupVars = utils_1.VarParser.parseVars(this.group);
        ccml.push.apply(ccml, groupVars[1]);
        ccml.push("<SAVEREC voiceMailName=\"" + groupVars[0] + "\" varName=\"___recordMessageId___\" errorActionId=\"" + id + "_onError\"/>");
        ccml.push("<WAIT nextActionId=\"" + id + "_onDone\"/>");
        ccml.push("<BOOKMARK id=\"" + id + "_onError\"/>");
        ccml.push.apply(ccml, this.events[exports.RecordMessageOnEvent.OnError].map(function (a) { return a.toCCML(); }));
        ccml.push("<WAIT nextActionId=\"" + id + "_onSkipDone\"/>");
        ccml.push("<BOOKMARK id=\"" + id + "_onDone\"/>");
        ccml.push("<SET varName=\"recordings\" value=\"${recordings}" + this.name + "=${___recordMessageId___}&amp;\" export=\"true\"/>");
        ccml.push(this.discardCCML());
        ccml.push.apply(ccml, this.events[exports.RecordMessageOnEvent.OnSave].map(function (a) { return a.toCCML(); }));
        ccml.push("<BOOKMARK id=\"" + id + "_onSkipDone\"/>");
        ccml.push(this.discardCCML());
        return ccml.join("");
    };
    return RecordMessage;
}(interfaces_1.CoreAction));
exports.RecordMessage = RecordMessage;
//# sourceMappingURL=RecordMessage.js.map