"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CCMLBuilder_1 = require("../CCMLBuilder");
var utils_1 = require("../utils");
var ActionFunction = (function () {
    function ActionFunction(data) {
        this._id = Math.floor(Math.random() * Number.MAX_SAFE_INTEGER).toString(36);
        this._name = data.name;
        this._args = data.args || {};
        this._actions = new CCMLBuilder_1.CCMLBuilder(data).getActions();
        this._returns = data.returns || {};
    }
    Object.defineProperty(ActionFunction.prototype, "id", {
        get: function () { return this._id; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionFunction.prototype, "name", {
        get: function () { return this._name; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionFunction.prototype, "args", {
        get: function () { return this._args; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionFunction.prototype, "actions", {
        get: function () { return this._actions.slice(0); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ActionFunction.prototype, "returns", {
        get: function () { return this._returns; },
        enumerable: true,
        configurable: true
    });
    ActionFunction.prototype.toCCML = function () {
        var _this = this;
        var actionString = this._actions.map(function (a) { return a.toCCML().replace(/___localVar_/g, "___funcVar_" + _this._name + "_"); }).join("");
        var returnMap = Object.keys(this._returns).reduce(function (p, k) {
            var exp = utils_1.VarParser.parseExpression(_this._returns[k]).replace(/___localVar_/g, "___funcVar_" + _this._name + "_");
            p += ", " + JSON.stringify(k) + ": " + exp;
            return p;
        }, "{ \"_id\": " + JSON.stringify(this._id) + ", \"_name\": " + JSON.stringify(this._name) + " ") + " }";
        return "\n            <!-- Function " + this._name + " -->\n            <SUBSCRIPT id=\"___func_" + this._name + "___\">\n                " + actionString + "\n                <SET varName=\"___actionFunctionData___\" evaluate=\"true\" export=\"false\"><![CDATA[ (function(){ return JSON.stringify(" + returnMap + "); }()) ]]></SET>\n            </SUBSCRIPT>\n        ";
    };
    return ActionFunction;
}());
exports.ActionFunction = ActionFunction;
function buildArgSets(name, args) {
    return Object.keys(args).reduce(function (p, k) {
        var exp = utils_1.VarParser.parseExpression(args[k]);
        return p += "<SET varName=\"___funcVar_" + name + "_" + k.replace(/^\$/, "") + "___\" evaluate=\"true\" export=\"false\"><![CDATA[ (function () { return JSON.stringify(" + exp + "); }()) ]]></SET>";
    }, "");
}
exports.buildArgSets = buildArgSets;
//# sourceMappingURL=ActionFunction.js.map